/*
 * Part of "Intel(R) AMT Management Interface" Linux driver 
 *
 * Copyright (c) 2003 - 2007 Intel Corp.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification.
 * 2. Redistributions in binary form must reproduce at minimum a disclaimer
 *    substantially similar to the "NO WARRANTY" disclaimer below
 *    ("Disclaimer") and any redistribution must be conditioned upon
 *    including a substantially similar Disclaimer requirement for further
 *    binary redistribution.
 * 3. Neither the names of the above-listed copyright holders nor the names
 *    of any contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * NO WARRANTY
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 */

#ifndef _HECI_H_
#define _HECI_H_

#include <linux/version.h>
#include <linux/spinlock.h>
#include <linux/list.h>
#include <linux/pci.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/module.h>
#include <linux/aio.h>
#include <linux/types.h>
#include "heci_data_structures.h"


extern const guid_t heci_pthi_guid;
extern const uint8_t start_wd_params[];
extern const uint8_t stop_wd_params[];

/**
 * memory IO BAR definition  
 */
#define     BAR_0                        0
#define     BAR_1                        1
#define     BAR_5                        5
/**
 * Number of queue lists used by this driver
 */
#define    PCI_HECI_DEVICE_ID1   0x2974
#define    PCI_HECI_DEVICE_ID2   0x2984
#define    PCI_HECI_DEVICE_ID3   0x2994
#define    PCI_HECI_DEVICE_ID4   0x29A4
#define    PCI_HECI_DEVICE_ID5   0x29C4

/**
 * heci init function prototypes
 */
iamt_heci_device_t *init_heci_device(struct pci_dev *pdev);
void heci_reset(iamt_heci_device_t * device_object, boolean_t interrupts);
int heci_hw_init(iamt_heci_device_t * device_object);
int heci_initialize_clients(void *data);
heci_file_private_t *alloc_priv(struct file *file);
int heci_disconnect_host_client(iamt_heci_device_t * device_object,
				heci_file_private_t * file_extension);
void heci_initialize_list(struct io_heci_list *list,
			  iamt_heci_device_t * device_object);
void heci_flush_list(struct io_heci_list *list,
		     heci_file_private_t * file_extension);
void heci_flush_queues(iamt_heci_device_t * device_object,
		       heci_file_private_t * file_extension);

void heci_remove_client_from_file_list(iamt_heci_device_t * device_object,
				       uint8_t host_client_id);

/**
 *  interrupt function prototype
 */
irqreturn_t heci_isr_interrupt(int irq, void *dev_id,
			       struct pt_regs *regs);
void heci_wd_timer(unsigned long data);
void heci_bh_handler(void *data);
/**
 *  input output function prototype
 */
int heci_ioctl_get_version(iamt_heci_device_t * device, int if_num,
			   struct heci_message_data *u_msg,
			   struct heci_message_data k_msg,
			   heci_file_private_t * file_extension);
int heci_ioctl_connect_client(iamt_heci_device_t * device, int if_num,
			      struct heci_message_data *u_msg,
			      struct heci_message_data k_msg,
			      struct file *file);
int heci_ioctl_wd(iamt_heci_device_t * device, int if_num,
		  struct heci_message_data k_msg,
		  heci_file_private_t * file_extension);
int legacy_ioctl_send_message(iamt_heci_device_t * device, int if_num,
			      struct heci_message_data k_msg,
			      struct file *file);
int legacy_ioctl_receive_message(iamt_heci_device_t * device, int if_num,
				 struct heci_message_data *u_msg,
				 struct heci_message_data k_msg,
				 struct file *file);
int heci_start_read(iamt_heci_device_t * device, int if_num,
		    heci_file_private_t * file_extension);
int pthi_write(iamt_heci_device_t * device,
	       struct heci_cb_private *kernel_priv_cb);
int pthi_read(iamt_heci_device_t * device, int if_num, struct file *file,
	      char *ubuf, size_t length);
void run_next_legacy_cmd(iamt_heci_device_t * device);

#endif				/* _HECI_H_ */
